/******************************************************************************
 *  #Include inc_IniReadWrite.AHK
 *
 *  Datum: 2005-05-06 14:27
 *  Autor: Magra19
 *  Vers:  0.09
 *  Typ:   Method
 *  AHK:   1.0.35+
 *  OS:    Win 2000
 *
 *  PnP:   no;
 *         user actions needed: search for: NextUserChange
 *
 ******************************************************************************
 *
 *  Benefits:
 *
 *  - INI file only written when changes detected
 *    (set externally by calling program = has to be user provided)
 *
 *  - saved after a user selectable time interval;
 *    default setting of the time interval is supplied
 *
 *  - can be kept for different purposes in different directories
 *    (see "where does the INI file exist ?"):
 *    1. per user
 *    2. per computer
 *    3. universal per network
 *
 *  - detects external changes of the INI file done by the user e.g. with
 *    an editor; rereads INI file to fetch the user changes
 *    but discards any changes detected by the program
 *
 *  - everything is kept in one INI file, no changes to the registry
 *    made. If you don't want to use this module any longer just
 *    delete the #Include and the INI file.
 *
 *  Call:
 *
 *    TimeInterval = 15000  ; this line omitted: default is 10 sec
 *    #Include inc_IniReadWrite.AHK
 *
 *  User has to provide two functions
 *  (see appended examples inc_IniReadWrite_User.AHK):
 *
 *    mIniRead_All_User() { ... }
 *    mIniWrite_All_User() { ... }
 *
 ******************************************************************************
 * where does the INI file exist ?
 *
 * 1.: dir in <This User>
 *     (every user has his own file)
 *     e.g. "C:\Documents and Settings\<Username>\ApplicationData\AutoHotkey\<Prog>.INI"
 *
 * 2.: dir in "All Users"
 *     (one file for all users of this computer)
 *     e.g. "C:\Documents and Settings\All Users\ApplicationData\AutoHotkey\<Prog>.INI"
 *
 * 3.: in %A_WorkingDir%
 *     (the dir that contain the (main) AHK file:
 *     universal setting if AHK file is e.g. in a mapped drive on a server.
 *     Remember: Writing for the INI file should be allowed)
 *     e.g. "U:\Programs\AHKs\<Prog>.INI" where U: could be the mapped drive to a server.
 *
 *  How to change the location of the INI file:
 *  If you want to change from e.g. 1. (per user) to 3. (universal) just copy the file
 *  into the new dir and delete the old one. (Copy not rename because the file inherits
 *  the NTFS access rights from the dir it is copied to.)
 *
 *  When two files e.g. in 1. and 2. exist testing for existens is started at location 1,
 *  then tests 2. and then 3. Testing stopps when the first file is detected.
 *  It is possible that some computers (all using the same AHK in a mapped drive)
 *  use the per user setting while others use the per computer setting (2.)
 *  or the universal (3.).
 *
 *  Please remember:
 *  ----------------
 *  Universal setting (3.) on a mapped drive means that several computers work
 *  on the same INI file.
 *  This also means (write rights provided) that any changes done by one computer
 *  in the INI file are re-read by all other computers !!
 *
 ******************************************************************************
 *
 *  With any changes of an INI value IniFile_Write_All should be set to TRUE
 *  this method should be called periodically (e.g. every 5 seconds)
 *  so not to overload the system with writing to the INI file
 *  (e.g. when the window is moved on the screen and the new coordinates
 *  should save these).
 *
 *  When the main program detects a change in these values it should set:
 *
 *    IniFile_Write_All := True
 *
 * ----------------------------------------------------------------------------
 *  Wenn das Hauptprogramm nderungen an den INI-Werten erkennt, sollte es
 *
 *    IniFile_Write_All := True
 *
 *  setzen. Die Schreibroutine sollten nicht zu hufig gerufen werden
 *  (z.B. alle 5 sek oder langfristiger), um den Schreibaufwand auf die
 *  Platte nicht zu hoch werden zu lassen (z.B. wenn ein Fenster auf dem
 *  Bildschirm bewegt wird).
 *
 ******************************************************************************
 *
 *  this function call is the only command that is executed
 *  in the auto-execute section !
 *  The following function definitions are skipped.
 * ----------------------------------------------------------------------------
 *  dieses ist das einzige Kommando, das im Kopfbereich des AHK-Files
 *  ausgefhrt wird. Die nachfolgenden Funktionsdefinitionen werden bersprungen.
 */

mIniFile_Init( TimeInterval )

/*
 ******************************************************************************
 */
mIniFile_Init( TimeInterval )
{
	Global

	If ( TimeInterval )
		IniFile_Write_TimeInterval := TimeInterval
	Else
		If ( ! IniFile_Write_TimeInterval )   ; if not defined before
			IniFile_Write_TimeInterval := 10000  ; default: 10 sec

	vTooltip := False
	IniFile_Write_All := False
	IniFile_Dir_SaveINI = AutoHotkey

	Local vProgram_NNam, Laenge
#IncludeAgain inc_vProgramName.AHK

	; find "ApplicationData" (only defined for "This User" not for "All Users")
	IniFile_Path = %APPDATA%
	StringSplit, IniFile_Array_, IniFile_Path , \, ,
	IniFile_Nr := IniFile_Array_0   ; Nr of the last element
	IniFile_ApplicationData := IniFile_Array_%IniFile_Nr%   ; contains only "ApplicationData"
	; construct ...\All Users\ApplicationData
	IniFile_ApplicationData_AllUsers = %ALLUSERSPROFILE%\%IniFile_ApplicationData%
	IniFile_ApplicationData_ThisUser = %APPDATA%

	IniFile_Path_SaveINI_AllUsers = %IniFile_ApplicationData_AllUsers%\%IniFile_Dir_SaveINI%
	IniFile_Path_SaveINI_ThisUser = %IniFile_ApplicationData_ThisUser%\%IniFile_Dir_SaveINI%
	IniFile_PathFile_SaveINI_AllUsers = %IniFile_Path_SaveINI_AllUsers%\%vProgram_NNam%.INI
	IniFile_PathFile_SaveINI_ThisUser = %IniFile_Path_SaveINI_ThisUser%\%vProgram_NNam%.INI
	; for debugging:
	;Tooltip %IniFile_PathFile_SaveINI_AllUsers%
	;sleep 5000
	;Tooltip %IniFile_PathFile_SaveINI_ThisUser%
	;sleep 5000

	IfExist %IniFile_PathFile_SaveINI_ThisUser%   ; 1.
	{
		IniFile_Save_File_in_Dir = 1
		IniFile_PathFile_SaveINI = %IniFile_PathFile_SaveINI_ThisUser%
		IniFile_Path             = %IniFile_Path_SaveINI_ThisUser%
	}
	Else IfExist %IniFile_PathFile_SaveINI_AllUsers%   ; 2.
	{
		IniFile_Save_File_in_Dir = 2
		IniFile_PathFile_SaveINI = %IniFile_PathFile_SaveINI_AllUsers%
		IniFile_Path             = %IniFile_Path_SaveINI_AllUsers%
	}
	Else IfExist %vProgram_NNam%.INI   ; 3.
	{
		IniFile_Save_File_in_Dir = 3
		IniFile_PathFile_SaveINI = %A_WorkingDir%\%vProgram_NNam%.INI
		IniFile_Path             = %A_WorkingDir%
	}
	If ! IniFile_Save_File_in_Dir   ; if not found in any location
	{
		IniFile_Save_File_in_Dir = 1
		IniFile_PathFile_SaveINI = %IniFile_PathFile_SaveINI_ThisUser%
		IniFile_Path             = %IniFile_Path_SaveINI_ThisUser%
		IniFile_Write_All        = True
		MsgBox INI file not found`n`nI created in new one in:`n`n%IniFile_PathFile_SaveINI%
	}

	StringSplit, IniFile_Array_, IniFile_PathFile_SaveINI , \, ,
	IniFile_Drive = %IniFile_Array_1%
	;Tooltip IniFile_Drive:`n`n%IniFile_Drive%`n`n%IniFile_PathFile_SaveINI%
	;Sleep 5000

	StringRight IniFile_PathFile_SaveINIRight, IniFile_PathFile_SaveINI, 60  ; for Debug purposes
	;Tooltip %IniFile_ScriptRight%
	FileCreateDir, %IniFile_Path%

	/*
	 * use external INI file for compiled scripts or for network installations:
	 * (If commented out: use external INI file.)
	 * /
	IfInString, vProgram_NNamExt, .ahk
		; IniFile_PathFile_SaveINI = %A_ScriptFullPath%
	Else
		IniFile_PathFile_SaveINI = %A_ScriptDir%\%vProgram_NNam%.INI
	/*
	...
	 */
	mIniRead_All()
	FileGetTime, IniFile_FileTime , %IniFile_PathFile_SaveINI%, M   ; Modified-Last
	IniFile_FileTime_Old = %IniFile_FileTime%

	SetTimer lbIniFile_Write, %IniFile_Write_TimeInterval%
Return 0

lbIniFile_Write:
	mIniWrite_All()
Return

}

/*
 *************************************************************************
 */

mIniWrite_All()
{
	Global

	;  file was altered from outside the program: maybe edited
	;  and saved by the user to change one of the values by
	;  an editor: then read instead of write (changes in this
	;  program are lost in this case)
	;  --- M=Modified A=Access-Last C=Creation
	FileGetTime, IniFile_FileTime , %IniFile_PathFile_SaveINI%, M
	; Tooltip mIniWrite_All:`n%IniFile_PathFile_SaveINI%`n%IniFile_FileTime_Old% : %IniFile_FileTime%
	If ( IniFile_FileTime_Old <> IniFile_FileTime )
	{
		mIniRead_All()
		IniFile_FileTime_Old = %IniFile_FileTime%
		Tooltip INI-File neu gelesen - wurde vom User beschrieben
		; Tooltip INI file was read - has been overwritten by the user
		Sleep 5000
	}
	Else
	{
 		; Tooltip  mIniWrite_All: INI-File will be written
		; Sleep 5000
		mIniWrite_All_User()
		/*********************************************************************************
		 * only if INI values are saved in AHK file !!
		 * and "#Include inc_AutoReLoad.AHK" is used.
	 	 * Writing of INI values should not be seen as modification
		 * modify only if values are saved !
	 	 * if INI files has been written: update time marker
		 */
		; If ( IniFile_PathFile_SaveINI = A_ScriptFullPath OR IniFile_Write_All )
		If ( IniFile_Write_All )
		{
			FileGetTime, IniFile_FileTime , %IniFile_PathFile_SaveINI%, M  ; Modified last
			IniFile_FileTime_Old := IniFile_FileTime
		}
		/*********************************************************************************
		 */
		IniFile_Write_All := False
		; Tooltip  mIniWrite_All: INI file has been written
		; Sleep 5000

	}

	Return 0
}

/*
 *************************************************************************
 */
mIniRead_All()
{
	Global
	mIniRead_All_User()
	If ( vTooltip )
	{
		Tooltip  mIniRead_All: INI file has been read
		Sleep 5000
	}

	Return
}
/*************************************************************************
 *   END of inc_IniReadWrite.AHK
 *************************************************************************
 */



/*************************************************************************
 *  <<<<< NextUserChange (and last)
 *  Template for your code; you will have to change this
 *************************************************************************
 *  #Include inc_IniReadWrite_User.AHK
 *
 *  Datum: 2005-05-06 14:27
 *  Autor: Magra19
 *  Typ:   Method
 *************************************************************************
 *
 *  this has to be a function declaration for use in #Include statement:
 *  /
mIniWrite_All_User()
{
	Global  ; don't forget: necessary to access global variables
	;
	; detect changes here (If ( Var <> Var_Old) ... IniFile_Write_All := True)
	; or in your main program.
	;
	; !!   Please :
	; !!   DO NOT reset IniFile_Write_All before leaving this method.
	; !!   The calling method needs this variable and will reset it.
	;
	If ! IniFile_Write_All
		Return

	; change the following lines to the variables you want to save !!!

	; ------------------------------------------------------------------------
	; [Settings]

	; IniDelete, %IniFile_PathFile_SaveINI%, Settings   ; also deletes possible comments !!

	IniWrite, %A_Now%, %IniFile_PathFile_SaveINI%, Settings, Date  ; only for documentation
	IniWrite, %vPath_StarsData1%, %IniFile_PathFile_SaveINI%, Settings, Path_StarsData
	IniWrite, %UsedList%, %IniFile_PathFile_SaveINI%, Settings, UsedList
	If ( CurrText <> "" )
		IniWrite, %CurrText%, %IniFile_PathFile_SaveINI%, Settings, CurrText

	; X and Y: blank if hidden
	If ( (GuiWindowX <> "") and (GuiWindowX > 10 ) )
		IniWrite, %GuiWindowX%, %IniFile_PathFile_SaveINI%, Settings, GuiWindowX

	If ( (GuiWindowY <> "") and (GuiWindowY > 10 ) )
		IniWrite, %GuiWindowY%, %IniFile_PathFile_SaveINI%, Settings, GuiWindowY

	IniWrite, %GuiWindowWidth%, %IniFile_PathFile_SaveINI%, Settings, GuiWindowWidth
	IniWrite, %GuiWindowHeight%, %IniFile_PathFile_SaveINI%, Settings, GuiWindowHeight

	; ------------------------------------------------------------------------
	; [WinSize]

	IniWrite, %AW_Control%, %IniFile_PathFile_SaveINI%, WinSize, AW_Control
	Loop, %AWNr_CntEntries%
	{
		AWNr_Name	  = AWNr_%A_Index%
		AWNr_Element := AWNr_%A_Index%
		IniWrite, %AWNr_Element%, %IniFile_PathFile_SaveINI%, WinSize, %AWNr_Name%
 		; Tooltip IniWrite: %AWNr_Name%: %AWNr_Element%
		; Sleep 2000
	}

	If ( vTooltip )
		Tooltip mIniWrite_All_User: has been written

Return
}
;
;***************************************************************************
;
mIniRead_All_User()
{
	Global

	; ------------------------------------------------------------------------
	; [Settings]

	IniRead, UsedList, %IniFile_PathFile_SaveINI%, Settings, UsedList
	IniRead, vPath_StarsData1, %IniFile_PathFile_SaveINI%, Settings, Path_StarsData
	If ( vPath_StarsData1 <> "" )
		If vPath_StarsData1 = ERROR
			vPath_StarsData1 = D:\StarsData\Root\UserData\
		vPath_StarsData = %vPath_StarsData1%

	IniRead, CurrText, %IniFile_PathFile_SaveINI%, Settings, CurrText
	IniRead, GuiWindowX, %IniFile_PathFile_SaveINI%, Settings, GuiWindowX
	IniRead, GuiWindowY, %IniFile_PathFile_SaveINI%, Settings, GuiWindowY
	IniRead, GuiWindowWidth, %IniFile_PathFile_SaveINI%, Settings, GuiWindowWidth
	IniRead, GuiWindowHeight, %IniFile_PathFile_SaveINI%, Settings, GuiWindowHeight

	IfEqual, UsedList, ERROR
		UsedList =

	GuiWindowXOld := GuiWindowX
	GuiWindowYOld := GuiWindowY
	GuiWindowWidthOld := GuiWindowWidth
	GuiWindowHeightOld := GuiWindowHeight

	; ------------------------------------------------------------------------
	; [WinSize]

	AW_Control_Old = %AW_Control%
	IniRead, AW_Control, %IniFile_PathFile_SaveINI%, WinSize, AW_Control
	If ( ! AW_Control or AW_Control = "ERROR" )   ; Blank oder ERROR
		vCtrl_Alt_Y_Hotkey = ^!Y
	Else
		vCtrl_Alt_Y_Hotkey = %AW_Control%
	If ( 	AW_Control_Old <> AW_Control )   ; wenn was geaendert wurde
		mCtrl_Alt_Y_Init( %vCtrl_Alt_Y_Time% )

	If ( AWNr_CntEntries <> 0 )
	{
		Nr = 0
		Loop
		{
			AWNr_Name	  = AWNr_%A_Index%
			AWNr_Element := AWNr_%A_Index%
			IniRead, AWNr_Element, %IniFile_PathFile_SaveINI%, WinSize, %AWNr_Name%
			If ( AWNr_Element = "ERROR" )
				Break
			If ( AWNr_Element <> "" )
			{
				Nr += 1
				AWNr_Name = AWNr_%Nr%
				AWNr_%Nr% = %AWNr_Element%
				AWNr__GID_%Index% := 0

				AWNr_CntEntries := Nr
				; Tooltip IniRead: %AWNr_CntEntries% | %AWNr_Name%: %AWNr_Element%
				; Sleep 200
			}
		}
	}

	If ( vTooltip )
		Tooltip mIniRead_All_User has been read

Return
}
/***************************************************************************
 *  END of #Include inc_IniReadWrite_User.AHK
 ***************************************************************************
 */

